<?php

/**
 * Provide a admin area servers view for the serverpilot module
 *
 * @link       	https://designedforpixels.com
 * @author		Gary Jordan (gary@designedforpixels.com)
 * @since      	1.0.0
 *
 * @package    	WP_Cloud_Server
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;

	$servers = get_option( 'wpcs_serverpilot_api_data', array() );

	if ( !isset( $servers['servers']['data'] ) ) {
		// Create instance of the Cloudways API
		$api				= new WP_Cloud_Server_ServerPilot_API();
		$servers['servers']	= $api->call_api( 'servers', null, false, 900, 'GET' );
	}

	$module_data	= get_option( 'wpcs_module_list' );			
	?>
<div class="uk-overflow-auto">
	<h2 class="uk-margin-remove-top uk-heading-divider"><?php _e( 'ServerPilot Servers', 'wp-cloud-server' ); ?></h2>
	<table class="uk-table uk-table-striped">
    	<thead>
        	<tr>
            	<th class="col-name"><?php esc_html_e( 'Server Name', 'wp-cloud-server' ); ?></th>
            	<th><?php esc_html_e( 'Plan', 'wp-cloud-server' ); ?></th>
            	<th><?php esc_html_e( 'Date Created', 'wp-cloud-server' ); ?></th>
            	<th><?php esc_html_e( 'Auto Install', 'wp-cloud-server' ); ?></th>
            	<!-- <th><?php esc_html_e( 'Sites', 'wp-cloud-server' ); ?></th> -->
            	<th><?php esc_html_e( 'PHP Version', 'wp-cloud-server' ); ?></th>
            	<!-- <th><?php esc_html_e( 'IP Address', 'wp-cloud-server' ); ?></th> -->
				<th><?php esc_html_e( 'Manage', 'wp-cloud-server' ); ?></th>
       	 </tr>
    	</thead>
    	<tbody>
			<?php
			if ( !empty( $servers['servers']['data'] ) ) {
				foreach ( $servers['servers']['data'] as $key => $server ) {
					
					// Obtain Key for desired PHP Runtime Version (ServerPilot occasionally changes the availability!)
					if ( is_array( $server['available_runtimes'] ) ) {
						$key = ( count( $server['available_runtimes'] ) - 1 );
						$php_version = $server['available_runtimes'][ $key ];
					} else {
						$php_version = 'php7.4';
					}

					?>
        			<tr>
						<td><?php echo $server['name']; ?></td>
						<?php $plan = ( $server['plan'] == 'first_class' ) ? 'First Class' : ucfirst($server['plan']); ?>
						<td><?php echo $plan; ?></td>
            			<td><?php echo date( 'd/M/Y', $server['datecreated']); ?></td>
						<td><?php echo ( 1 == $server['autoupdates'] ) ? 'Enabled' : 'Not Enabled'; ?></td>
						<!-- <td><?php echo '0'; ?></td> -->
						<td><?php echo preg_replace( '/^php/', 'PHP ', $php_version ); ?></td>
						<!-- <td><?php echo $server['lastaddress']; ?></td> -->
						<td><a class="uk-link" href="#managed-server-modal-<?php echo $server['id']; ?>" uk-toggle>Manage</a></td>
        			</tr>
					<?php
				}
			} else {
			?>
					<tr>
						<td colspan="7"><?php esc_html_e( 'No Server Information Available', 'wp-cloud-server' ) ?></td>
					</tr>
			<?php
			}
			?>
    	</tbody>
	</table>
</div>

	<?php
	if ( !empty( $servers['servers']['data'] ) ) {
		foreach ( $servers['servers']['data'] as $key => $server ) {
		
			// Obtain Key for desired PHP Runtime Version (ServerPilot occasionally changes the availability!)
			if ( is_array( $server['available_runtimes'] ) ) {
				$key = ( count( $server['available_runtimes'] ) - 1 );
				$php_version = $server['available_runtimes'][ $key ];
			} else {
				$php_version = 'php7.4';
			}
				    ?>

			        <div id="managed-server-modal-<?php echo $server['id']; ?>" uk-modal>
    			        <div class="server-modal uk-modal-dialog uk-modal-body">
					        <button class="uk-modal-close-default" type="button" uk-close></button>
        			        <h2><?php esc_html_e( 'Manage Server: ', 'wp-cloud-server' ); ?><span style="color: #A78BFA;"><?php echo $server['name']; ?></span></h2>
					        <hr class="clear">
					        <div class="server-info uk-modal-body" uk-overflow-auto>
						        <table class="server-info uk-table uk-table-striped">
    						        <tbody>
										 <tr>
            						<td><?php esc_html_e( 'Server Name', 'wp-cloud-server' ); ?></td>
            						<td><?php echo "{$server['name']}"; ?></td>
       							</tr>
        						<tr>
            						<td><?php esc_html_e( 'Plan', 'wp-cloud-server' ); ?></td>
            						<?php $plan = ( $server['plan'] == 'first_class' ) ? 'First Class' : ucfirst($server['plan']); ?>
									<td><?php echo $plan; ?></td>
        						</tr>
        						<tr>
            						<td><?php esc_html_e( 'Auto Updates', 'wp-cloud-server' ); ?></td>
           							<td><?php echo ( 1 == $server['autoupdates'] ) ? 'Enabled' : 'Not Enabled'; ?></td>
       	 						</tr>
        						<tr>
            						<td><?php esc_html_e( 'PHP Version', 'wp-cloud-server' ); ?></td>
           	 						<td><?php echo preg_replace( '/^php/', 'PHP ', $php_version ); ?></td>
        						</tr>
        						<tr>
            						<td><?php esc_html_e( 'IP Address', 'wp-cloud-server' ); ?></td>
            						<td><?php echo isset($server['lastaddress']) ? $server['lastaddress'] : 'Not Available'; ?></td>
        						</tr>
        						<tr>
            						<td><?php esc_html_e( 'Server ID', 'wp-cloud-server' ); ?></td>
            						<td><?php echo "{$server['id']}"; ?></td>
        						</tr>
        						<tr>
            						<td><?php esc_html_e( 'Date Created', 'wp-cloud-server' ); ?></td>
            						<td><?php echo date( 'd/M/Y', $server['datecreated']); ?></td>
        						</tr>
    						        </tbody>
						        </table>
					        </div>
					        <hr>
					        <a class="uk-button uk-button-danger uk-align-left uk-margin-remove-bottom" href="#delete-server-<?php echo $server['id']; ?>" uk-toggle><?php esc_attr_e( 'DELETE', 'wp-cloud-server' ) ?></a>
    			        </div>
			        </div>
					<div id="delete-server-<?php echo $server['id']; ?>" uk-modal>
    			<div class="server-modal uk-modal-dialog uk-modal-body">
        			<div class="uk-modal-body">
						<p class="uk-text-lead"><?php esc_html_e( "Take care! This will delete '{$server['name']}' from your ServerPilot account! It cannot be reversed!", 'wp-cloud-server' ); ?></p>
            			<p><?php esc_html_e( "Please confirm by clicking the 'CONFIRM' button below!", 'wp-cloud-server' ); ?></p>
        			</div>
					<div class="uk-modal-footer uk-text-right">
						<form method="post" action="<?php echo admin_url( 'admin-post.php' ) ?>">
							<input type="hidden" name="action" value="handle_serverpilot_server_action">
							<input type="hidden" name="wpcs_serverpilot_server_action" value="delete">
							<input type="hidden" name="wpcs_serverpilot_server_id" value="<?php echo $server['id'];?>">
							<?php wp_nonce_field( 'handle_serverpilot_server_action_nonce', 'wpcs_handle_serverpilot_server_action_nonce' ); ?>
							<div class="uk-button-group uk-margin-remove-bottom">
								<a class="uk-button uk-button-default uk-margin-small-right" href="#managed-server-modal-<?php echo $server['id']; ?>" uk-toggle><?php esc_attr_e( 'CANCEL', 'wp-cloud-server' ) ?></a>
            					<?php wpcs_submit_button( 'Confirm Delete', 'danger', 'delete_server', false ); ?>
							</div>
						</form>
					</div>
    			</div>
			</div>
    		<?php
		}
	}